------------------------------------------------------------------------------
--  Test: Access_Control_001
--  Verification: This test verifies that the owner of a object or schema,
--      always have access to a given object
--
--  Test Details:
--      1.  An account will be created and denied access to an object
--      2.  Denied access will then be verified
--      3.  The account will be make owner of the schema or object
--      4.  For each of the ownership cases above the test will verify 
--             access is allowed
--
--  Test Dependenices:
--      1.  This test must be executed as a sysadmin
------------------------------------------------------------------------------

DECLARE @error_message 	nvarchar(1024)

BEGIN TRY
	SET NOCOUNT ON

	------------------------------------------------------------------------------
	-- If the version is not correct then do not run the script
	------------------------------------------------------------------------------
	IF SERVERPROPERTY( N'ProductVersion' ) <> N'10.50.2500.0'
	BEGIN
		SET @error_message = N'TESTRESULT: FAIL [Access_Control_001] This test can only be executed on SQL Server 2008 R2 SP1';
		PRINT @error_message
		RETURN;
	END

	------------------------------------------------------------------------------
	-- Verify test dependencies
	------------------------------------------------------------------------------
	IF ( IS_SRVROLEMEMBER( N'sysadmin' ) = 0 )
	BEGIN
		SET @error_message = N'TESTRESULT: FAIL [Access_Control_001] This test must be executed by a sysadmin';
		PRINT @error_message
		RETURN;
	END

	------------------------------------------------------------------------------
	-- Create account and test objects
	------------------------------------------------------------------------------
	CREATE USER [USR_Access_Control_001] WITHOUT LOGIN;
	EXEC ( N'CREATE SCHEMA [SCH_Access_Control_001]' );
	CREATE TABLE [SCH_Access_Control_001].[TBL_Access_Control_001] ( c1 int );
	INSERT [SCH_Access_Control_001].[TBL_Access_Control_001] VALUES ( 1 );

	------------------------------------------------------------------------------
	-- Deny access on the table to the account
	------------------------------------------------------------------------------
	DENY SELECT ON [SCH_Access_Control_001].[TBL_Access_Control_001] TO [USR_Access_Control_001];

	------------------------------------------------------------------------------
	-- Verify access to the object is denied
	------------------------------------------------------------------------------
	BEGIN TRY
		EXEC( N'SELECT * FROM [SCH_Access_Control_001].[TBL_Access_Control_001]' ) AS USER = N'USR_Access_Control_001';
	END TRY
	BEGIN CATCH
		--------------------------------------------------------------------------------------------------------
		-- Expect 229: Generic Permission Denied Message
		--------------------------------------------------------------------------------------------------------
		IF ( error_number() <> 229 )
		BEGIN
			SET @error_message = N'TESTRESULT: FAIL [Access_Control_001] Expect error 229 when accessing table as non-owner';
			PRINT @error_message
			RETURN;
		END		
	END CATCH

	------------------------------------------------------------------------------
	-- Make account owner of the object
	------------------------------------------------------------------------------
	ALTER AUTHORIZATION ON OBJECT::[SCH_Access_Control_001].[TBL_Access_Control_001] TO [USR_Access_Control_001]

	------------------------------------------------------------------------------
	-- Verify access to the object is allowed
	------------------------------------------------------------------------------
	BEGIN TRY
		EXEC( N'SELECT * FROM [SCH_Access_Control_001].[TBL_Access_Control_001]' ) AS USER = N'USR_Access_Control_001';
	END TRY
	BEGIN CATCH
		SET @error_message = N'TESTRESULT: FAIL [Access_Control_001] Expect no error when accessing table as object owner';
		PRINT @error_message
		RETURN;
	END CATCH

	------------------------------------------------------------------------------
	-- Make account owner of the schema
	------------------------------------------------------------------------------
	ALTER AUTHORIZATION ON OBJECT::[SCH_Access_Control_001].[TBL_Access_Control_001] TO SCHEMA OWNER
	ALTER AUTHORIZATION ON SCHEMA::[SCH_Access_Control_001] TO [USR_Access_Control_001]

	------------------------------------------------------------------------------
	-- Verify access to the object is allowed
	------------------------------------------------------------------------------
	BEGIN TRY
		EXEC( N'SELECT * FROM [SCH_Access_Control_001].[TBL_Access_Control_001]' ) AS USER = N'USR_Access_Control_001';
	END TRY
	BEGIN CATCH
		SET @error_message = N'TESTRESULT: FAIL [Access_Control_001] Expect no error when accessing table as schema owner';
		PRINT @error_message
		RETURN;
	END CATCH

	PRINT 'TESTRESULT: PASS [Access_Control_001]';
END TRY
BEGIN CATCH

	SET @error_message = N'TESTRESULT: FAIL [Access_Control_001] ' + error_message();
	PRINT @error_message

END CATCH
go

--------------------------------------------------------------------------------------------------------
-- Cleanup ALL accounts and objects
--------------------------------------------------------------------------------------------------------
IF ( OBJECT_ID( N'SCH_Access_Control_001.TBL_Access_Control_001' ) IS NOT NULL )
BEGIN
	DROP TABLE [SCH_Access_Control_001].[TBL_Access_Control_001];
END
go

IF ( SCHEMA_ID( N'SCH_Access_Control_001' ) IS NOT NULL )
BEGIN
	DROP SCHEMA [SCH_Access_Control_001];
END
go

IF ( USER_ID( N'USR_Access_Control_001' ) IS NOT NULL )
BEGIN
	DROP USER [USR_Access_Control_001];
END
go
