------------------------------------------------------------------------------
--  Test: Access_Control_003
--  Verification: This test verifies permission check is allowed when granted 
--      and not allowed if denied
--
--  Test Details:
--      1.  An account will be created and granted select on a table
--      2.  Select access will then be verified
--      3.  Select permission on the table will be denied
--      4.  Select access will then be verified (should not be allowed)
--
--  Test Dependenices:
--      1.  This test must be executed as a sysadmin
------------------------------------------------------------------------------

DECLARE @error_message 	nvarchar(1024)

BEGIN TRY
	SET NOCOUNT ON

	------------------------------------------------------------------------------
	-- If the version is not correct then do not run the script
	------------------------------------------------------------------------------
	IF SERVERPROPERTY( N'ProductVersion' ) <> N'10.50.2500.0'
	BEGIN
		SET @error_message = N'TESTRESULT: FAIL [Access_Control_003] This test can only be executed on SQL Server 2008 R2 SP1';
		PRINT @error_message
		RETURN;
	END

	------------------------------------------------------------------------------
	-- Verify test dependencies
	------------------------------------------------------------------------------
	IF ( IS_SRVROLEMEMBER( N'sysadmin' ) = 0 )
	BEGIN
		SET @error_message = N'TESTRESULT: FAIL [Access_Control_003] This test must be executed by a sysadmin';
		PRINT @error_message
		RETURN;
	END

	------------------------------------------------------------------------------
	-- Create account and test objects
	------------------------------------------------------------------------------
	CREATE USER [USR_Access_Control_003] WITHOUT LOGIN;
	EXEC ( N'CREATE SCHEMA [SCH_Access_Control_003]' );
	CREATE TABLE [SCH_Access_Control_003].[TBL_Access_Control_003] ( c1 int );
	INSERT [SCH_Access_Control_003].[TBL_Access_Control_003] VALUES ( 1 );

	------------------------------------------------------------------------------
	-- Grant Select permission on the table to the account
	------------------------------------------------------------------------------
	GRANT SELECT ON [SCH_Access_Control_003].[TBL_Access_Control_003] TO [USR_Access_Control_003];

	------------------------------------------------------------------------------
	-- Verify access to the object is denied
	------------------------------------------------------------------------------
	BEGIN TRY
		EXEC( N'SELECT * FROM [SCH_Access_Control_003].[TBL_Access_Control_003]' ) AS USER = N'USR_Access_Control_003';
	END TRY
	BEGIN CATCH
		--------------------------------------------------------------------------------------------------------
		-- Expect No Error
		--------------------------------------------------------------------------------------------------------
		SET @error_message = N'TESTRESULT: FAIL [Access_Control_003] Expect no error when selecting with select permission on the table';
		PRINT @error_message
		RETURN;
	END CATCH

	------------------------------------------------------------------------------
	-- Revoke Select permission on the table to the account
	-- Grant Select permission on the table schema
	------------------------------------------------------------------------------
	DENY SELECT ON [SCH_Access_Control_003].[TBL_Access_Control_003] TO [USR_Access_Control_003];

	------------------------------------------------------------------------------
	-- Verify access to the object is not allowed
	------------------------------------------------------------------------------
	BEGIN TRY
		EXEC( N'SELECT * FROM [SCH_Access_Control_003].[TBL_Access_Control_003]' ) AS USER = N'USR_Access_Control_003';
	END TRY
	BEGIN CATCH
		--------------------------------------------------------------------------------------------------------
		-- Expect 229: Generic Permission Denied Message
		--------------------------------------------------------------------------------------------------------
		IF ( error_number() <> 229 )
		BEGIN
			SET @error_message = N'TESTRESULT: FAIL [Access_Control_003] Expect error 229 when select permission is denied';
			PRINT @error_message
			RETURN;
		END		
	END CATCH

	PRINT 'TESTRESULT: PASS [Access_Control_003]';
END TRY
BEGIN CATCH

	SET @error_message = N'TESTRESULT: FAIL [Access_Control_003] ' + error_message();
	PRINT @error_message

END CATCH
go

--------------------------------------------------------------------------------------------------------
-- Cleanup ALL accounts and objects
--------------------------------------------------------------------------------------------------------
IF ( OBJECT_ID( N'SCH_Access_Control_003.TBL_Access_Control_003' ) IS NOT NULL )
BEGIN
	DROP TABLE [SCH_Access_Control_003].[TBL_Access_Control_003];
END
go

IF ( SCHEMA_ID( N'SCH_Access_Control_003' ) IS NOT NULL )
BEGIN
	DROP SCHEMA [SCH_Access_Control_003];
END
go

IF ( USER_ID( N'USR_Access_Control_003' ) IS NOT NULL )
BEGIN
	DROP USER [USR_Access_Control_003];
END
go
