------------------------------------------------------------------------------
--  Test: Auditing_001
--  Verification: This test verifies that the default Common Criteria trace
--      is started and that a sample event is properly audited
--
--  Test Details:
--      1.  An account will be created 
--      2.  The account will attempt to grant a permission without the
--             ability to do so
--      3.  The Common Criteria trace will be checked for the appropriate
--             events
--
--  Test Dependenices:
--      1.  This test must be executed as a sysadmin
------------------------------------------------------------------------------

DECLARE @error_message 	nvarchar(1024)

BEGIN TRY
	SET NOCOUNT ON
	DECLARE @trace_path   nvarchar(4000)
	DECLARE @trace_id     int

	------------------------------------------------------------------------------
	-- If the version is not correct then do not run the script
	------------------------------------------------------------------------------
	IF SERVERPROPERTY( N'ProductVersion' ) <> N'10.50.2500.0'
	BEGIN
		SET @error_message = N'TESTRESULT: FAIL [Auditing_001] This test can only be executed on SQL Server 2008 R2 SP1';
		PRINT @error_message
		RETURN;
	END

	------------------------------------------------------------------------------
	-- Verify test dependencies
	------------------------------------------------------------------------------
	IF ( IS_SRVROLEMEMBER( N'sysadmin' ) = 0 )
	BEGIN
		SET @error_message = N'TESTRESULT: FAIL [Auditing_001] This test must be executed by a sysadmin';
		PRINT @error_message
		RETURN;
	END

	------------------------------------------------------------------------------
	-- Verify default Common Criteria trace is available
	------------------------------------------------------------------------------
	IF ( ( SELECT count(*) FROM sys.traces WHERE path LIKE '%cc_trace_%' ) <> 1 )
	BEGIN
		SET @error_message = N'TESTRESULT: FAIL [Auditing_001] The default Common Criteria trace does not exist';
		PRINT @error_message
		RETURN;
	END

	SELECT @trace_path = path, @trace_id = id FROM sys.traces WHERE path LIKE '%cc_trace_%';

	------------------------------------------------------------------------------
	-- Create account and test objects
	------------------------------------------------------------------------------
	CREATE USER [USR_Auditing_001] WITHOUT LOGIN;

	------------------------------------------------------------------------------
	-- Attempt to grant a database level permission without permission to do so
	------------------------------------------------------------------------------
	BEGIN TRY
		EXEC( N'GRANT ALTER ANY APPLICATION ROLE TO [public]' ) AS USER = N'USR_Auditing_001';
	END TRY
	BEGIN CATCH
		--------------------------------------------------------------------------------------------------------
		-- Expect 4613: Grantor does not have GRANT permission
		--------------------------------------------------------------------------------------------------------
		IF ( error_number() <> 4613 )
		BEGIN
			SET @error_message = N'TESTRESULT: FAIL [Auditing_001] Expect error 4613 granting without permission to do so';
			PRINT @error_message
			RETURN;
		END		
	END CATCH


	------------------------------------------------------------------------------
	-- Verify the event is in the trace
	------------------------------------------------------------------------------
	IF (
	( SELECT count(*) FROM ::fn_trace_gettable( @trace_path, default ) 
	     WHERE
		--
		-- Verify the StartTime is within a reasonable range of time for simple verification
		--
		DATEDIFF( second, StartTime, GETDATE() ) < 3
		AND DBUserName = 'USR_Auditing_001'
		AND EventClass = 102 -- Audit Database Scope GDR Event
		AND Success = 0
	) <> 1 )
	BEGIN
		SET @error_message = N'TESTRESULT: FAIL [Auditing_001] Event 102 was not fired properly';
		PRINT @error_message
		RETURN;
	END
	
	PRINT 'TESTRESULT: PASS [Auditing_001]';
END TRY
BEGIN CATCH

	SET @error_message = N'TESTRESULT: FAIL [Auditing_001] ' + error_message();
	PRINT @error_message

END CATCH
go

--------------------------------------------------------------------------------------------------------
-- Cleanup ALL accounts and objects
--------------------------------------------------------------------------------------------------------
IF ( USER_ID( N'USR_Auditing_001' ) IS NOT NULL )
BEGIN
	DROP USER [USR_Auditing_001];
END
go
