------------------------------------------------------------------------------
--  Test: Auditing_002
--  Verification: This test verifies that the default Common Criteria trace
--      is started and that a sample event is properly audited
--
--  Test Details:
--      1.  An account will be created 
--      2.  The account will attempt to grant a permission without the
--             ability to do so
--      3.  The Common Criteria trace will be checked for the appropriate
--             events
--
--  Test Dependenices:
--      1.  This test must be executed as a sysadmin
------------------------------------------------------------------------------

DECLARE @error_message 	nvarchar(1024)

BEGIN TRY
	SET NOCOUNT ON
	DECLARE @trace_path   nvarchar(4000)
	DECLARE @trace_id     int

	------------------------------------------------------------------------------
	-- If the version is not correct then do not run the script
	------------------------------------------------------------------------------
	IF SERVERPROPERTY( N'ProductVersion' ) <> N'10.50.2500.0'
	BEGIN
		SET @error_message = N'TESTRESULT: FAIL [Auditing_002] This test can only be executed on SQL Server 2008 R2 SP1';
		PRINT @error_message
		RETURN;
	END

	------------------------------------------------------------------------------
	-- Verify test dependencies
	------------------------------------------------------------------------------
	IF ( IS_SRVROLEMEMBER( N'sysadmin' ) = 0 )
	BEGIN
		SET @error_message = N'TESTRESULT: FAIL [Auditing_002] This test must be executed by a sysadmin';
		PRINT @error_message
		RETURN;
	END

	------------------------------------------------------------------------------
	-- Verify default Common Criteria trace is available
	------------------------------------------------------------------------------
	IF ( ( SELECT count(*) FROM sys.traces WHERE path LIKE '%cc_trace_%' ) <> 1 )
	BEGIN
		SET @error_message = N'TESTRESULT: FAIL [Auditing_002] The default Common Criteria trace does not exist';
		PRINT @error_message
		RETURN;
	END

	SELECT @trace_path = path, @trace_id = id FROM sys.traces WHERE path LIKE '%cc_trace_%';

	------------------------------------------------------------------------------
	-- Create accounts and test objects
	------------------------------------------------------------------------------
	CREATE USER [USR_Auditing_002A] WITHOUT LOGIN;
	CREATE USER [USR_Auditing_002B] WITHOUT LOGIN;

	------------------------------------------------------------------------------
	-- Set a filter on the Common Criteria trace so only user "A" is audited
	------------------------------------------------------------------------------
	DECLARE @retval int

	SET @retval = 1
	EXEC @retval = sp_trace_setstatus @trace_id, 0;
	IF ( @retval <> 0 )
	BEGIN
		SET @error_message = N'TESTRESULT: FAIL [Session_Handling_006] Failure stopping default Common Criteria trace';
		PRINT @error_message
		RETURN;
	END

	SET @retval = 1
	EXEC @retval = sp_trace_setfilter @trace_id, 40 /* DBUserName */, 0 /* AND */, 0 /* EQUALS */, N'USR_Auditing_002A';
	IF ( @retval <> 0 )
	BEGIN
		SET @error_message = N'TESTRESULT: FAIL [Session_Handling_006] Failure executing sp_trace_setfilter';
		PRINT @error_message
		RETURN;
	END

	SET @retval = 1
	EXEC @retval = sp_trace_setstatus @trace_id, 1;
	IF ( @retval <> 0 )
	BEGIN
		SET @error_message = N'TESTRESULT: FAIL [Session_Handling_006] Failure starting default Common Criteria trace';
		PRINT @error_message
		RETURN;
	END

	------------------------------------------------------------------------------
	-- Attempt to create a role without permission to do so (first user)
	------------------------------------------------------------------------------
	BEGIN TRY
		EXEC( N'CREATE ROLE [RL_Auditing_002A]' ) AS USER = N'USR_Auditing_002A';
	END TRY
	BEGIN CATCH
		--------------------------------------------------------------------------------------------------------
		-- Expect 15247: generic permission error
		--------------------------------------------------------------------------------------------------------
		IF ( error_number() <> 15247 )
		BEGIN
			SET @error_message = N'TESTRESULT: FAIL [Auditing_002] Expect generic permission error 15247';
			PRINT @error_message
			RETURN;
		END		
	END CATCH

	------------------------------------------------------------------------------
	-- Attempt to create a role without permission to do so (first user)
	------------------------------------------------------------------------------
	BEGIN TRY
		EXEC( N'CREATE ROLE [RL_Auditing_002B]' ) AS USER = N'USR_Auditing_002B';
	END TRY
	BEGIN CATCH
		--------------------------------------------------------------------------------------------------------
		-- Expect 15247: generic permission error
		--------------------------------------------------------------------------------------------------------
		IF ( error_number() <> 15247 )
		BEGIN
			SET @error_message = N'TESTRESULT: FAIL [Auditing_002] Expect generic permission error 15247';
			PRINT @error_message
			RETURN;
		END		
	END CATCH

	------------------------------------------------------------------------------
	-- Verify the event is in the trace
	------------------------------------------------------------------------------
	IF (
	( SELECT count(*) FROM ::fn_trace_gettable( @trace_path, default ) 
	     WHERE
		--
		-- Verify the StartTime is within a reasonable range of time for simple verification
		--
		DATEDIFF( second, StartTime, GETDATE() ) < 5
		AND EventClass = 130 -- Audit Database Principal Management Event
		AND Success = 0
	) <> 1 )
	BEGIN
		SET @error_message = N'TESTRESULT: FAIL [Auditing_002] Event 102 was not fired properly';
		PRINT @error_message
		RETURN;
	END
	
	------------------------------------------------------------------------------
	-- Remove all filters from the default Common Criteria trace
	------------------------------------------------------------------------------
	SET @retval = 1
	EXEC @retval = sp_trace_setstatus @trace_id, 0;
	IF ( @retval <> 0 )
	BEGIN
		SET @error_message = N'TESTRESULT: FAIL [Session_Handling_006] Failure stopping default Common Criteria trace';
		PRINT @error_message
		RETURN;
	END

	SET @retval = 1
	EXEC @retval = sp_trace_setfilter @traceid = @trace_id, @columnid = NULL;
	IF ( @retval <> 0 )
	BEGIN
		SET @error_message = N'TESTRESULT: FAIL [Session_Handling_006] Failure executing sp_trace_setfilter';
		PRINT @error_message
		RETURN;
	END

	SET @retval = 1
	EXEC @retval = sp_trace_setstatus @trace_id, 1;
	IF ( @retval <> 0 )
	BEGIN
		SET @error_message = N'TESTRESULT: FAIL [Session_Handling_006] Failure starting default Common Criteria trace';
		PRINT @error_message
		RETURN;
	END

	------------------------------------------------------------------------------
	-- Attempt to create a role without permission to do so (first user)
	------------------------------------------------------------------------------
	WAITFOR DELAY '00:00:05'

	BEGIN TRY
		EXEC( N'CREATE ROLE [RL_Auditing_002A]' ) AS USER = N'USR_Auditing_002A';
	END TRY
	BEGIN CATCH
		--------------------------------------------------------------------------------------------------------
		-- Expect 15247: generic permission error
		--------------------------------------------------------------------------------------------------------
		IF ( error_number() <> 15247 )
		BEGIN
			SET @error_message = N'TESTRESULT: FAIL [Auditing_002] Expect generic permission error 15247';
			PRINT @error_message
			RETURN;
		END		
	END CATCH

	------------------------------------------------------------------------------
	-- Attempt to create a role without permission to do so (first user)
	------------------------------------------------------------------------------
	BEGIN TRY
		EXEC( N'CREATE ROLE [RL_Auditing_002B]' ) AS USER = N'USR_Auditing_002B';
	END TRY
	BEGIN CATCH
		--------------------------------------------------------------------------------------------------------
		-- Expect 15247: generic permission error
		--------------------------------------------------------------------------------------------------------
		IF ( error_number() <> 15247 )
		BEGIN
			SET @error_message = N'TESTRESULT: FAIL [Auditing_002] Expect generic permission error 15247';
			PRINT @error_message
			RETURN;
		END		
	END CATCH

	------------------------------------------------------------------------------
	-- Verify both events are in the trace
	------------------------------------------------------------------------------
	IF (
	( SELECT count(*) FROM ::fn_trace_gettable( @trace_path, default ) 
	     WHERE
		--
		-- Verify the StartTime is within a reasonable range of time for simple verification
		--
		DATEDIFF( second, StartTime, GETDATE() ) < 5
		AND EventClass = 130 -- Audit Database Principal Management Event
		AND Success = 0
	) <> 2 )
	BEGIN
		SET @error_message = N'TESTRESULT: FAIL [Auditing_002] Event 102 was not fired properly';
		PRINT @error_message
		RETURN;
	END

	PRINT 'TESTRESULT: PASS [Auditing_002]';
END TRY
BEGIN CATCH

	SET @error_message = N'TESTRESULT: FAIL [Auditing_002] ' + error_message();
	PRINT @error_message

END CATCH
go

--------------------------------------------------------------------------------------------------------
-- Cleanup ALL accounts and objects
--------------------------------------------------------------------------------------------------------
IF ( USER_ID( N'USR_Auditing_002A' ) IS NOT NULL )
BEGIN
	DROP USER [USR_Auditing_002A];
END
go

IF ( USER_ID( N'USR_Auditing_002B' ) IS NOT NULL )
BEGIN
	DROP USER [USR_Auditing_002B];
END
go